##################################################################
#
# (C) Copyright 2002 Kapil Thangavelu <kvthan@wm.edu>
# All Rights Reserved
#
# This file is part of CMFDeployment.
#
# CMFDeployment is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# CMFDeployment is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################

"""
Purpose: Renders Content For Deployment
Author: kapil thangavelu <kvthan@wm.edu> @2002
License: GPL
Created: 8/10/2002
CVS: $Id: ContentMastering.py,v 1.17 2002/10/18 23:49:13 kapil Exp $
"""

from Namespace import *

from DeploymentExceptions import InvalidSkinName
from MimeMapping import MimeMappingContainer, getMimeExprContext
from utils import file2string

from Log import LogFactory

log = LogFactory('Mastering')

class ContentMastering(Folder):

    meta_type = 'Content Mastering' 

    manage_options = (

        {'label':'Overview',
         'action':'overview'},
        
        {'label':'Skin',
         'action':'skin'},

#        {'label':'SiteRoot',
#         'action':'root'},        

        {'label':'MimeMapping',
         'action':'mime/manage_main'},        

        {'label':'Policy',
         'action':'../overview'}

        )

    overview = DTMLFile('ui/ContentMasteringOverview', globals())
    skin     = DTMLFile('ui/ContentMasteringSkin', globals())
    root     = DTMLFile('ui/ContentMasteringRoot', globals())

    def __init__(self, id):
        
        self.id = id
        self.skn_name = None

    def editRoot(self, enable, server, server_path, port, protocol, RESPONSE=None):
        """ """
        
        self.site_root.edit(enable, server, server_path, protocol, port)

        if RESPONSE is not None:
            RESPONSE.redirect('overview')

    def editSkin(self, enable, skin_name, RESPONSE=None):
        """ """
        
        skin_tool = getToolByName(self, 'portal_skins')
        skin = skin_tool.getSkinByName(skin_name.strip())
        
        if skin is None:
            raise InvalidSkinName("%s is not a valid skin"%str(skin_name))

        self.site_skin.edit(enable, skin_name)

        if RESPONSE is not None:
            RESPONSE.redirect('overview')

    def manage_afterAdd(self, item, container):

        ob = MimeMappingContainer('mime')
        self._setObject('mime', ob)

        #        ob = SiteChainRoot()
        #        self._setObject('site_root', ob)

        ob = SiteChainSkin()
        self._setObject('site_skin', ob)

    #################################
    def prepare(self, descriptor):

        portal = getToolByName(self, 'portal_url').getPortalObject()        

        c = descriptor.getContent()
        ctx = getMimeExprContext(c, portal) 

        mappings = self.mime.objectValues()
                
        for m in mappings:
            
            if m.valid(ctx):
                
                descriptor.setExtension(m.extension(ctx))
                
                if m.ghost:
                    descriptor.setRenderMethod(None) # xxx redundant
                    descriptor.setGhost(1)
                else:
                    descriptor.setRenderMethod(m.view_method)
                    
                return 1
            
        return None

    def cook(self, descriptor):

        if descriptor.isGhost():
            return
        
        c = descriptor.getContent()
        vm = descriptor.getRenderMethod()

        if vm is None: # XXX redundant
            descriptor.setGhost(1)
            return
        
        render = getattr(c, vm, None) or c.__call__

        log.debug('rendering %s %s %s'%(c.getId(), vm, str(render)))

        try: 
            if getattr(aq_base(render), 'isDocTemp', 0):
                descriptor.setRendered(apply(render, (self, self.REQUEST)))
            elif hasattr(aq_base(c), 'precondition'): # test if its a file object
                descriptor.setRendered(file2string(c))
                descriptor.setBinary(1)
            else:
                descriptor.setRendered(render())
        except:
            log.error('Error While Rendering %s'%(str(c.getPhysicalPath())))
            descriptor.setGhost(1) # ghostify it

    #################################
    def setup(self):
        #pass
        #self.site_root.lock()
        self.site_skin.lock()
        
    def tearDown(self):
        #pass
        #self.site_root.unlock()
        self.site_skin.unlock()

#################################
# both the chain classes are hacks

class SiteChainSkin(SimpleItem):
    """

    changes the skin mid request,

    """

    meta_type = 'Site Chain Skin'
    id = 'site_skin'
    
    _v_active = None
    _v_saved_skin_data = None

    
    def __init__(self):
        self.enable = None
        self.skin_name = None

    def edit(self, enable, skin_name):
        
        self.enable = not not int(enable)
        self.skin_name = skin_name

    def lock(self):

        if not self.enable or self._v_active or not self.skin_name:
            print 'aborting chain skin'
            return

        portal = getToolByName(self, 'portal_url').getPortalObject()
        req = getattr(self, 'REQUEST', {})
        self._v_saved_skin_data = portal._v_skindata
        portal._v_skindata = (req,
                              self.getSkinByName(self.skin_name),
                              {})
        self._v_active = 1
        
    def unlock(self):

        if not self._v_active:
            return

        portal = getToolByName(self, 'portal_url').getPortalObject()
        portal._v_skindata = self._v_saved_skin_data

        self._v_saved_skin_data = None
        self._v_active = None
        

class SiteChainRoot(SimpleItem):
    """
    it creates a site root temporarily

    for now i'm not bothering to define an unlock,
    because this is way too hackish to safely be
    certain of unlocking, and it has no byproducts
    beyond the current request.
    
    """
    meta_type = 'Site Chain Root'
    id = 'site_root'

    _v_active = None
    
    def __init__(self):
        
        self.enable = None # flag to use or not
        self.protocol = None
        self.port = None
        self.server = None        
        self.server_path = None
        
    def edit(self, enable, server, server_path, protocol, port):
        
        self.enable = not not int(enable)
        self.server = server.strip()
        self.server_path = server_path.strip()
        self.protocol = protocol.strip()
        self.port = port.strip()

    def lock(self):

        if not self.enable or not self.server or not self.server_path or self._v_active:
            print 'aborting chain root'
            return

        # XXX This is the voodoo
        req = self.REQUEST
        ob = self.getContentOrganization().getActiveStructure().getCMFMountPoint()
        
        other = req.other
        path  = filter(None, self.server_path.split('/'))


        req.setServerURL(protocol=self.protocol, hostname=self.server, port=self.port)
        req._script[:] = path
        del req._steps[:]
        other['VirtualRootPhysicalPath'] = ob.getPhysicalPath()
                       
        req._resetURLS()

        self._v_active = 1
        
        return

    def unlock(self):

        if not self.active:
            return

        # XXX
        
        self._v_active = None
        
        return
        
            
            
