##################################################################
#
# (C) Copyright 2002 Kapil Thangavelu <kvthan@wm.edu>
# All Rights Reserved
#
# This file is part of CMFDeployment.
#
# CMFDeployment is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# CMFDeployment is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
Author: kapil thangavelu <kvthan@wm.edu>
License: GPL
CVS: $Id: DeploymentHistory.py,v 1.5 2002/10/18 23:49:13 kapil Exp $
Purpose: Simple History storage... to be expanded

"""
from Namespace import *
from time import time
from BTrees.OIBTree import OISet
from BTrees.Length import Length

class DeploymentHistoryContainer(Folder):

    all_meta_types = ()
    
    meta_type = 'Deployment History Container'

    manage_options = (
        
        {'label':'History',
         'action':'history_overview'},

        {'label':'Policy',
         'action':'../overview'},
        )

    security = ClassSecurityInfo()

    history_overview = DTMLFile('ui/DeploymentHistoryOverview', globals())

    def __init__(self, id):
        self.id = id
        self._records = OISet()
        self._record_length = Length(0)

    security.declarePrivate('addHistory')
    def addHistory(self):
        self._records.insert(DeploymentHistory())

    security.declareProtected(Permissions.view_management_screens, 'getHistories')
    def getHistories(self):
        return tuple(self._records)

    security.declareProtected(Permissions.view_management_screens, 'getLastEntry')
    def getLastEntry(self):
        
        if self._record_length() == 0:
            return DeploymentHistory()
        
        return self._records[self._record_length()]


InitializeClass(DeploymentHistoryContainer)

class DeploymentHistory(SimpleItem):

    meta_type = 'Deployment History'

    manage_options = ()

    security = ClassSecurityInfo()    

    def __init__(self):
        self.creation_time = DateTime()
        self.id = str(int(self.creation_time.timeTime()))
        
InitializeClass(DeploymentHistory)

    
