##################################################################
#
# (C) Copyright 2002 Kapil Thangavelu <kvthan@wm.edu>
# All Rights Reserved
#
# This file is part of CMFDeployment.
#
# CMFDeployment is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# CMFDeployment is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
Purpose: Organizes Content in a Deployment Target Structure
Author: kapil thangavelu <kvthan@wm.edu>
License: GPL
Created: 8/10/2002
CVS: $Id: DeploymentPolicy.py,v 1.12 2002/10/18 19:30:34 kapil Exp $
"""

import DefaultConfiguration
from Namespace import *
from Descriptor import ContentDescriptor

from DeploymentInterfaces import IDeploymentPolicy
import DeploymentStrategy

class DeploymentPolicy(Folder):

    meta_type = 'Deployment Policy'

    __implements__ = IDeploymentPolicy

    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.ManagePortal)

    manage_options = (

        {'label':'overview',
         'action':'overview'},
        
        {'label':'Identification',
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentIdentification},

        {'label':'Organization',
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentOrganization},

        {'label':'Skins',
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentDirectoryViews},

        {'label':'URIs',
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentURIs},

        {'label':'Templating',
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentMastering},

        {'label':'Deployment', 
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentDeployment},

        {'label':'History',
         'action':'%s/manage_workspace'%DefaultConfiguration.DeploymentHistory},

        {'label':'Strategies',
         'action':'%s/manage_workspace'%DefaultConfiguration.DeploymentStrategy},

        {'label':'Tool', 
         'action':'../manage_main'}
        
        ) 

    overview = DTMLFile('ui/PolicyOverview', globals())
    identification = DTMLFile('ui/ContentIdentification', globals())
    _active = 1
    
    icon = 'misc_/CMFDeployment/policy.png'
    
    def __init__(self, id):
        self.id = id 

    def getContentIdentification(self):
        return self._getOb(DefaultConfiguration.ContentIdentification)

    def getContentOrganization(self):
        return self._getOb(DefaultConfiguration.ContentOrganization)

    def getContentMastering(self):
        return self._getOb(DefaultConfiguration.ContentMastering)

    def getContentDeployment(self):
        return self._getOb(DefaultConfiguration.ContentDeployment)

    def getContentDirectoryViews(self):
        return self._getOb(DefaultConfiguration.ContentDirectoryViews)

    def getDeploymentHistory(self):
        return self._getOb(DefaultConfiguration.DeploymentHistory)

    def getDeploymentStrategy(self):
        return self._getOb(DefaultConfiguration.DeploymentStrategy)

    def getDeploymentURIs(self):
        return self._getOb(DefaultConfiguration.ContentURIs)

    def getDeploymentPolicy(self):
        return self

    def setActive(self, flag, RESPONSE=None):
        self._active = not not flag
        if RESPONSE is not None:
            RESPONSE.redirect('overview')

    def isActive(self):
        return self._active

    def execute(self, RESPONSE):
        """ """

        self._active = 1
        if not self.isActive():
            return
        
        strategy = self.getDeploymentStrategy().getStrategy()
        display = strategy(self)
        
        return display

    def manage_afterAdd(self, item, container):
        import DefaultConfiguration
        DefaultConfiguration.install(self)
    
InitializeClass(DeploymentPolicy)
    

