##################################################################
#
# (C) Copyright 2002 Kapil Thangavelu <kvthan@wm.edu>
# All Rights Reserved
#
# This file is part of CMFDeployment.
#
# CMFDeployment is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# CMFDeployment is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
 Purpose: read an xml serialization of a deployment policy into python objects
 Author: kapil t.
 CVS: $Id: PolicyReader.py,v 1.7 2002/10/18 19:30:34 kapil Exp $

"""

from xml.sax import make_parser, ContentHandler
from UserDict import UserDict

marker = []
class PolicyNode(UserDict):
    reserved = ('data',)

    def __getattr__(self, name):

        v = self.__dict__.get(name, marker)
        if v is marker:
            return self.data[name]
        return v
    
    def __setattr__(self, name, value):
        if name in PolicyNode.reserved:
            self.__dict__[name]=value
        else:
            self.data[name]=value
    
class MetaReader(ContentHandler):

    def __init__(self):

        self.buf = []
        self.prefix = ''
        self.stack = []
        
    def startElement(self, element_name, attrs):
        name = element_name.lower()
        if self.prefix: name = '%s%s'%(self.prefix, name.capitalize())      
        method = getattr(self, 'start%s'%name.capitalize(), None)

        # get rid of unicode
        d = {}
        for k, v in attrs.items():
            d[str(k)]= str(v)
        
        if method: 
            apply(method, (d,))

    def endElement(self, element_name):
        chars = str(''.join(self.buf)).strip()
        self.buf = []

        name = element_name.lower()
        
        if self.prefix: name = '%s%s'%(self.prefix, name.capitalize())            
        method = getattr(self, 'end%s'%name.capitalize(), None)
        
        if method: 
            apply(method, (chars,))
         
    def characters(self, chars):
        self.buf.append(chars)


class PolicyReader(MetaReader):

    def startDeploymentpolicy(self, attrs):
        self.policy = PolicyNode(attrs)

    def startIdentification(self, attrs):
        self.policy.ident = PolicyNode(attrs)
        self.prefix = 'Ident'

    def endIdentidentification(self, chars):
        self.prefix =''

    def startIdentfilter(self, attrs):
        filters = self.policy.ident.setdefault('filters', [])
        filter = PolicyNode(attrs)
        filters.append(filter)

    def startOrganization(self, attrs):
        organization = PolicyNode(attrs)
        organization['restricted']=[]
        self.policy.organization = organization
        self.prefix='Organization'

    def startOrganizationrestricted(self, attrs):
        restricteds = self.policy.organization.setdefault('restricted', [])
        restricteds.append(attrs['id'])

    def endOrganizationorganization(self, chars):
        self.prefix=''
        
    def startMastering(self, attrs):
        mastering = PolicyNode(attrs)
        self.policy.mastering = mastering
        self.prefix='Mastering'

    def endMastering(self, chars):
        self.prefix=''
        
    def startMasteringmime(self, attrs):
        mimes = self.policy.mastering.setdefault('mimes', [])
        mime = PolicyNode(attrs)
        mimes.append(mime)
        
    def getPolicy(self):
        return self.policy
        
def read_policy(file):
    # file = file handle
    parser = make_parser()
    reader = PolicyReader()
    parser.setContentHandler(reader)
    parser.parse(file)
    return reader.getPolicy()

def make_policy(portal, policy_node):

    import DefaultConfiguration
    
    deployment_tool = portal.portal_deployment
    deployment_tool.addPolicy(policy_node.id, policy_node.id)
    policy = getattr(deployment_tool, policy_node.id)
    
    identification = getattr(policy, DefaultConfiguration.ContentIdentification)
    for f in policy_node.ident.filters:
        identification.filters.addFilter(f.id, f.expr)
        
    mastering = getattr(policy, DefaultConfiguration.ContentMastering)
    
    for m in policy_node.mastering.mimes:
        mastering.mime.addMimeMapping(m.id,
                                      m.ext_expr,
                                      m.filter_expr,
                                      m.view_method,
                                      m.setdefault('ghost',0)
                                      )
                                                   
                                                   

    organization = getattr(policy, DefaultConfiguration.ContentOrganization)
    
    organization.structure.setMountPoint(policy_node.organization.fs_mount)
    organization.structure.setCMFMountPoint(policy_node.organization.cmf_mount)
    
    if policy_node.organization.has_key('restricted'):
        organization.structure.setRestrictedPoints(policy_node.organization.restricted)

    return policy
    
if __name__=='__main__':

    import sys
    import pprint
    
    policy = read_policy(sys.argv[1])

    pprint.pprint(policy)

    
